/*
 *  OS.cc
 */

#include <OS.h>
#include "my_be.h"


static VALUE
rb_kill_thread(VALUE self, VALUE varg1)
{
  thread_id arg1;
  status_t result;

  arg1 = NUM2LONG(varg1);
  result = kill_thread(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_resume_thread(VALUE self, VALUE varg1)
{
  thread_id arg1;
  status_t result;

  arg1 = NUM2LONG(varg1);
  result = resume_thread(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_suspend_thread(VALUE self, VALUE varg1)
{
  thread_id arg1;
  status_t result;

  arg1 = NUM2LONG(varg1);
  result = suspend_thread(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_rename_thread(VALUE self, VALUE varg1, VALUE varg2)
{
  thread_id arg1;
  char *arg2;
  status_t result;

  arg1 = NUM2LONG(varg1);
  arg2 = STR2CSTR(varg2);
  result = rename_thread(arg1, arg2);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_set_thread_priority(VALUE self, VALUE varg1, VALUE varg2)
{
  thread_id arg1;
  int32 arg2;
  status_t result;

  arg1 = NUM2LONG(varg1);
  arg2 = NUM2LONG(varg2);
  result = set_thread_priority(arg1, arg2);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_exit_thread(VALUE self, VALUE varg1)
{
  status_t arg1;

  arg1 = NUM2LONG(varg1);
  exit_thread(arg1);
  return Qnil;
}


static VALUE
rb_wait_for_thread(VALUE self, VALUE varg1)
{
  thread_id arg1;
  status_t arg2 = 0, result;
  VALUE vresult;

  arg1 = NUM2LONG(varg1);
  result = wait_for_thread(arg1, &arg2);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  vresult = LONG2NUM(arg2);
  return vresult;
}


static VALUE
rb_find_thread(int argc, VALUE *argv, VALUE self)
{
  thread_id result;
  VALUE varg1, vresult;

  switch (rb_scan_args(argc, argv, "01", &varg1)) {
    case 0:
      result = find_thread(NULL);
      break;

    case 1:
      {
        char *arg1 = NULL;

        if (!NIL_P(varg1)) {
          arg1 = STR2CSTR(varg1);
        }
        result = find_thread(arg1);
      }
      break;

    default:
      rb_bug("rb_find_thread");
  }
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_snooze(VALUE self, VALUE varg1)
{
  bigtime_t arg1;
  status_t result;

  arg1 = NUM2LLONG(varg1);
  result = snooze(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_snooze_until(VALUE self, VALUE varg1, VALUE varg2)
{
  bigtime_t arg1;
  int arg2;
  status_t result;

  arg1 = NUM2LLONG(varg1);
  arg2 = NUM2LONG(varg2);
  result = snooze_until(arg1, arg2);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_kill_team(VALUE self, VALUE varg1)
{
  team_id arg1;
  status_t result;

  arg1 = NUM2LONG(varg1);
  result = kill_team(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_is_computer_on(VALUE self)
{
  int32 result;
  VALUE vresult;

  result = is_computer_on();
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_is_computer_on_fire(VALUE self)
{
  double result;
  VALUE vresult;

  result = is_computer_on_fire();
  vresult = rb_float_new(result);
  return vresult;
}


static VALUE
rb_real_time_clock(VALUE self)
{
  uint32 result;
  VALUE vresult;

  result = real_time_clock();
  vresult = ULONG2NUM(result);
  return vresult;
}


static VALUE
rb_set_real_time_clock(VALUE self, VALUE varg1)
{
  int32 arg1;

  arg1 = NUM2LONG(varg1);
  set_real_time_clock(arg1);
  return Qnil;
}


static VALUE
rb_real_time_clock_usecs(VALUE self)
{
  bigtime_t result;
  VALUE vresult;

  result = real_time_clock_usecs();
  vresult = LLONG2NUM(result);
  return vresult;
}


static VALUE
rb_set_timezone(VALUE self, VALUE varg1)
{
  char *arg1;
  status_t result;

  arg1 = STR2CSTR(varg1);
  result = set_timezone(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_system_time(VALUE self)
{
  bigtime_t result;
  VALUE vresult;

  result = system_time();
  vresult = LLONG2NUM(result);
  return vresult;
}


static VALUE
rb_debugger(VALUE self, VALUE varg1)
{
  char *arg1;

  arg1 = STR2CSTR(varg1);
  debugger(arg1);
  return Qnil;
}


static VALUE
rb_disable_debugger(VALUE self, VALUE varg1)
{
  int arg1, result;
  VALUE vresult;

  arg1 = NUM2LONG(varg1);
  result = disable_debugger(arg1);
  vresult = LONG2NUM(result);
  return vresult;
}


void
Init_OS()
{
  rb_define_module_function(mBe, "kill_thread", RUBY_METHOD_FUNC(rb_kill_thread), 1);
  rb_define_module_function(mBe, "resume_thread", RUBY_METHOD_FUNC(rb_resume_thread), 1);
  rb_define_module_function(mBe, "suspend_thread", RUBY_METHOD_FUNC(rb_suspend_thread), 1);
  rb_define_module_function(mBe, "rename_thread", RUBY_METHOD_FUNC(rb_rename_thread), 2);
  rb_define_module_function(mBe, "set_thread_priority", RUBY_METHOD_FUNC(rb_set_thread_priority), 2);
  rb_define_module_function(mBe, "exit_thread", RUBY_METHOD_FUNC(rb_exit_thread), 1);
  rb_define_module_function(mBe, "wait_for_thread", RUBY_METHOD_FUNC(rb_wait_for_thread), 1);

  rb_define_module_function(mBe, "find_thread", RUBY_METHOD_FUNC(rb_find_thread), -1);

  rb_define_module_function(mBe, "snooze", RUBY_METHOD_FUNC(rb_snooze), 1);
  rb_define_module_function(mBe, "snooze_until", RUBY_METHOD_FUNC(rb_snooze_until), 2);

  rb_define_module_function(mBe, "kill_team", RUBY_METHOD_FUNC(rb_kill_team), 1);

  rb_define_module_function(mBe, "is_computer_on", RUBY_METHOD_FUNC(rb_is_computer_on), 0);
  rb_define_module_function(mBe, "is_computer_on_fire", RUBY_METHOD_FUNC(rb_is_computer_on_fire), 0);

  rb_define_module_function(mBe, "real_time_clock", RUBY_METHOD_FUNC(rb_real_time_clock), 0);
  rb_define_module_function(mBe, "set_real_time_clock", RUBY_METHOD_FUNC(rb_set_real_time_clock), 1);
  rb_define_module_function(mBe, "real_time_clock_usecs", RUBY_METHOD_FUNC(rb_real_time_clock_usecs), 0);
  rb_define_module_function(mBe, "set_timezone", RUBY_METHOD_FUNC(rb_set_timezone), 1);

  rb_define_module_function(mBe, "system_time", RUBY_METHOD_FUNC(rb_system_time), 0);

  rb_define_module_function(mBe, "debugger", RUBY_METHOD_FUNC(rb_debugger), 1);
  rb_define_module_function(mBe, "disable_debugger", RUBY_METHOD_FUNC(rb_disable_debugger), 1);


  rb_define_const(mBe, "OS_NAME_LENGTH", ULONG2NUM(B_OS_NAME_LENGTH));
  rb_define_const(mBe, "PAGE_SIZE", ULONG2NUM(B_PAGE_SIZE));
  rb_define_const(mBe, "INFINITE_TIMEOUT", ULLONG2NUM(B_INFINITE_TIMEOUT));

  rb_define_const(mBe, "NO_LOCK", ULONG2NUM(B_NO_LOCK));
  rb_define_const(mBe, "LAZY_LOCK", ULONG2NUM(B_LAZY_LOCK));
  rb_define_const(mBe, "FULL_LOCK", ULONG2NUM(B_FULL_LOCK));
  rb_define_const(mBe, "CONTIGUOUS", ULONG2NUM(B_CONTIGUOUS));
  rb_define_const(mBe, "LOMEM", ULONG2NUM(B_LOMEM));

  rb_define_const(mBe, "ANY_ADDRESS", ULONG2NUM(B_ANY_ADDRESS));
  rb_define_const(mBe, "EXACT_ADDRESS", ULONG2NUM(B_EXACT_ADDRESS));
  rb_define_const(mBe, "BASE_ADDRESS", ULONG2NUM(B_BASE_ADDRESS));
  rb_define_const(mBe, "CLONE_ADDRESS", ULONG2NUM(B_CLONE_ADDRESS));
  rb_define_const(mBe, "ANY_KERNEL_ADDRESS", ULONG2NUM(B_ANY_KERNEL_ADDRESS));

  rb_define_const(mBe, "READ_AREA", ULONG2NUM(B_READ_AREA));
  rb_define_const(mBe, "WRITE_AREA", ULONG2NUM(B_WRITE_AREA));

  rb_define_const(mBe, "CAN_INTERRUPT", ULONG2NUM(B_CAN_INTERRUPT));
  rb_define_const(mBe, "DO_NOT_RESCHEDULE", ULONG2NUM(B_DO_NOT_RESCHEDULE));
  rb_define_const(mBe, "CHECK_PERMISSION", ULONG2NUM(B_CHECK_PERMISSION));
  rb_define_const(mBe, "TIMEOUT", ULONG2NUM(B_TIMEOUT));
  rb_define_const(mBe, "RELATIVE_TIMEOUT", ULONG2NUM(B_RELATIVE_TIMEOUT));
  rb_define_const(mBe, "ABSOLUTE_TIMEOUT", ULONG2NUM(B_ABSOLUTE_TIMEOUT));

  rb_define_const(mBe, "ONE_SHOT_ABSOLUTE_ALARM", ULONG2NUM(B_ONE_SHOT_ABSOLUTE_ALARM));
  rb_define_const(mBe, "ONE_SHOT_RELATIVE_ALARM", ULONG2NUM(B_ONE_SHOT_RELATIVE_ALARM));
  rb_define_const(mBe, "PERIODIC_ALARM", ULONG2NUM(B_PERIODIC_ALARM));

  rb_define_const(mBe, "THREAD_RUNNING", ULONG2NUM(B_THREAD_RUNNING));
  rb_define_const(mBe, "THREAD_READY", ULONG2NUM(B_THREAD_READY));
  rb_define_const(mBe, "THREAD_RECEIVING", ULONG2NUM(B_THREAD_RECEIVING));
  rb_define_const(mBe, "THREAD_ASLEEP", ULONG2NUM(B_THREAD_ASLEEP));
  rb_define_const(mBe, "THREAD_SUSPENDED", ULONG2NUM(B_THREAD_SUSPENDED));
  rb_define_const(mBe, "THREAD_WAITING", ULONG2NUM(B_THREAD_WAITING));

  rb_define_const(mBe, "LOW_PRIORITY", ULONG2NUM(B_LOW_PRIORITY));
  rb_define_const(mBe, "NORMAL_PRIORITY", ULONG2NUM(B_NORMAL_PRIORITY));
  rb_define_const(mBe, "DISPLAY_PRIORITY", ULONG2NUM(B_DISPLAY_PRIORITY));
  rb_define_const(mBe, "URGENT_DISPLAY_PRIORITY", ULONG2NUM(B_URGENT_DISPLAY_PRIORITY));
  rb_define_const(mBe, "REAL_TIME_DISPLAY_PRIORITY", ULONG2NUM(B_REAL_TIME_DISPLAY_PRIORITY));
  rb_define_const(mBe, "URGENT_PRIORITY", ULONG2NUM(B_URGENT_PRIORITY));
  rb_define_const(mBe, "REAL_TIME_PRIORITY", ULONG2NUM(B_REAL_TIME_PRIORITY));

  rb_define_const(mBe, "SYSTEM_TIMEBASE", ULONG2NUM(B_SYSTEM_TIMEBASE));

  rb_define_const(mBe, "SYSTEM_TEAM", ULONG2NUM(B_SYSTEM_TEAM));

  rb_define_const(mBe, "MAX_CPU_COUNT", ULONG2NUM(B_MAX_CPU_COUNT));

  rb_define_const(mBe, "CPU_PPC_601", ULONG2NUM(B_CPU_PPC_601));
  rb_define_const(mBe, "CPU_PPC_603", ULONG2NUM(B_CPU_PPC_603));
  rb_define_const(mBe, "CPU_PPC_603e", ULONG2NUM(B_CPU_PPC_603e));
  rb_define_const(mBe, "CPU_PPC_604", ULONG2NUM(B_CPU_PPC_604));
  rb_define_const(mBe, "CPU_PPC_604e", ULONG2NUM(B_CPU_PPC_604e));
  rb_define_const(mBe, "CPU_PPC_750", ULONG2NUM(B_CPU_PPC_750));
  rb_define_const(mBe, "CPU_PPC_686", ULONG2NUM(B_CPU_PPC_686));
  rb_define_const(mBe, "CPU_AMD_29K", ULONG2NUM(B_CPU_AMD_29K));
  rb_define_const(mBe, "CPU_X86", ULONG2NUM(B_CPU_X86));
  rb_define_const(mBe, "CPU_MC6502", ULONG2NUM(B_CPU_MC6502));
  rb_define_const(mBe, "CPU_Z80", ULONG2NUM(B_CPU_Z80));
  rb_define_const(mBe, "CPU_ALPHA", ULONG2NUM(B_CPU_ALPHA));
  rb_define_const(mBe, "CPU_MIPS", ULONG2NUM(B_CPU_MIPS));
  rb_define_const(mBe, "CPU_HPPA", ULONG2NUM(B_CPU_HPPA));
  rb_define_const(mBe, "CPU_M68K", ULONG2NUM(B_CPU_M68K));
  rb_define_const(mBe, "CPU_ARM", ULONG2NUM(B_CPU_ARM));
  rb_define_const(mBe, "CPU_SH", ULONG2NUM(B_CPU_SH));
  rb_define_const(mBe, "CPU_SPARC", ULONG2NUM(B_CPU_SPARC));

  rb_define_const(mBe, "CPU_INTEL_X86", ULONG2NUM(B_CPU_INTEL_X86));
  rb_define_const(mBe, "CPU_INTEL_PENTIUM", ULONG2NUM(B_CPU_INTEL_PENTIUM));
  rb_define_const(mBe, "CPU_INTEL_PENTIUM75", ULONG2NUM(B_CPU_INTEL_PENTIUM75));
  rb_define_const(mBe, "CPU_INTEL_PENTIUM_486_OVERDRIVE", ULONG2NUM(B_CPU_INTEL_PENTIUM_486_OVERDRIVE));
  rb_define_const(mBe, "CPU_INTEL_PENTIUM_MMX", ULONG2NUM(B_CPU_INTEL_PENTIUM_MMX));
  rb_define_const(mBe, "CPU_INTEL_PENTIUM_MMX_MODEL_4", ULONG2NUM(B_CPU_INTEL_PENTIUM_MMX_MODEL_4));
  rb_define_const(mBe, "CPU_INTEL_PENTIUM_MMX_MODEL_8", ULONG2NUM(B_CPU_INTEL_PENTIUM_MMX_MODEL_8));
  rb_define_const(mBe, "CPU_INTEL_PENTIUM75_486_OVERDRIVE", ULONG2NUM(B_CPU_INTEL_PENTIUM75_486_OVERDRIVE));
  rb_define_const(mBe, "CPU_INTEL_PENTIUM_PRO", ULONG2NUM(B_CPU_INTEL_PENTIUM_PRO));
  rb_define_const(mBe, "CPU_INTEL_PENTIUM_II", ULONG2NUM(B_CPU_INTEL_PENTIUM_II));
  rb_define_const(mBe, "CPU_INTEL_PENTIUM_II_MODEL_3", ULONG2NUM(B_CPU_INTEL_PENTIUM_II_MODEL_3));
  rb_define_const(mBe, "CPU_INTEL_PENTIUM_II_MODEL_5", ULONG2NUM(B_CPU_INTEL_PENTIUM_II_MODEL_5));
  rb_define_const(mBe, "CPU_INTEL_CELERON", ULONG2NUM(B_CPU_INTEL_CELERON));
  rb_define_const(mBe, "CPU_INTEL_PENTIUM_III", ULONG2NUM(B_CPU_INTEL_PENTIUM_III));
  rb_define_const(mBe, "CPU_INTEL_PENTIUM_III_MODEL_8", ULONG2NUM(B_CPU_INTEL_PENTIUM_III_MODEL_8));

  rb_define_const(mBe, "CPU_AMD_X86", ULONG2NUM(B_CPU_AMD_X86));
  rb_define_const(mBe, "CPU_AMD_K5_MODEL0", ULONG2NUM(B_CPU_AMD_K5_MODEL0));
  rb_define_const(mBe, "CPU_AMD_K5_MODEL1", ULONG2NUM(B_CPU_AMD_K5_MODEL1));
  rb_define_const(mBe, "CPU_AMD_K5_MODEL2", ULONG2NUM(B_CPU_AMD_K5_MODEL2));
  rb_define_const(mBe, "CPU_AMD_K5_MODEL3", ULONG2NUM(B_CPU_AMD_K5_MODEL3));

  rb_define_const(mBe, "CPU_AMD_K6_MODEL6", ULONG2NUM(B_CPU_AMD_K6_MODEL6));
  rb_define_const(mBe, "CPU_AMD_K6_MODEL7", ULONG2NUM(B_CPU_AMD_K6_MODEL7));

  rb_define_const(mBe, "CPU_AMD_K6_MODEL8", ULONG2NUM(B_CPU_AMD_K6_MODEL8));
  rb_define_const(mBe, "CPU_AMD_K6_2", ULONG2NUM(B_CPU_AMD_K6_2));

  rb_define_const(mBe, "CPU_AMD_K6_MODEL9", ULONG2NUM(B_CPU_AMD_K6_MODEL9));
  rb_define_const(mBe, "CPU_AMD_K6_III", ULONG2NUM(B_CPU_AMD_K6_III));

  rb_define_const(mBe, "CPU_AMD_ATHLON_MODEL1", ULONG2NUM(B_CPU_AMD_ATHLON_MODEL1));

  rb_define_const(mBe, "CPU_CYRIX_X86", ULONG2NUM(B_CPU_CYRIX_X86));
  rb_define_const(mBe, "CPU_CYRIX_GXm", ULONG2NUM(B_CPU_CYRIX_GXm));
  rb_define_const(mBe, "CPU_CYRIX_6x86MX", ULONG2NUM(B_CPU_CYRIX_6x86MX));

  rb_define_const(mBe, "CPU_IDT_X86", ULONG2NUM(B_CPU_IDT_X86));
  rb_define_const(mBe, "CPU_IDT_WINCHIP_C6", ULONG2NUM(B_CPU_IDT_WINCHIP_C6));
  rb_define_const(mBe, "CPU_IDT_WINCHIP_2", ULONG2NUM(B_CPU_IDT_WINCHIP_2));

  rb_define_const(mBe, "CPU_RISE_X86", ULONG2NUM(B_CPU_RISE_X86));
  rb_define_const(mBe, "CPU_RISE_mP6", ULONG2NUM(B_CPU_RISE_mP6));

  rb_define_const(mBe, "CPU_X86_VENDOR_MASK", ULONG2NUM(B_CPU_X86_VENDOR_MASK));

  rb_define_const(mBe, "BEBOX_PLATFORM", ULONG2NUM(B_BEBOX_PLATFORM));
  rb_define_const(mBe, "MAC_PLATFORM", ULONG2NUM(B_MAC_PLATFORM));
  rb_define_const(mBe, "AT_CLONE_PLATFORM", ULONG2NUM(B_AT_CLONE_PLATFORM));
  rb_define_const(mBe, "ENIAC_PLATFORM", ULONG2NUM(B_ENIAC_PLATFORM));
  rb_define_const(mBe, "APPLE_II_PLATFORM", ULONG2NUM(B_APPLE_II_PLATFORM));
  rb_define_const(mBe, "CRAY_PLATFORM", ULONG2NUM(B_CRAY_PLATFORM));
  rb_define_const(mBe, "LISA_PLATFORM", ULONG2NUM(B_LISA_PLATFORM));
  rb_define_const(mBe, "TI_994A_PLATFORM", ULONG2NUM(B_TI_994A_PLATFORM));
  rb_define_const(mBe, "TIMEX_SINCLAIR_PLATFORM", ULONG2NUM(B_TIMEX_SINCLAIR_PLATFORM));
  rb_define_const(mBe, "ORAC_1_PLATFORM", ULONG2NUM(B_ORAC_1_PLATFORM));
  rb_define_const(mBe, "HAL_PLATFORM", ULONG2NUM(B_HAL_PLATFORM));
  rb_define_const(mBe, "BESM_6_PLATFORM", ULONG2NUM(B_BESM_6_PLATFORM));
  rb_define_const(mBe, "MK_61_PLATFORM", ULONG2NUM(B_MK_61_PLATFORM));
  rb_define_const(mBe, "NINTENDO_64_PLATFORM", ULONG2NUM(B_NINTENDO_64_PLATFORM));
}
