/*
 *  fs_index.cc
 */

#include <fs_index.h>
#include <errno.h>
#include "my_be.h"


static VALUE cIndexDir;


static void
close_index_dir(DIR *dir)
{
  if (dir) {
    int result;
    result = fs_close_index_dir(dir);
    if (result < 0) {
      DEBUG_TRACE();
      rb_warn("%s", strerror(errno));
    }
  }
}


static VALUE
rb_fs_close_index_dir(VALUE self, VALUE varg1)
{
  DIR *arg1;
  int result;

  Get_BInstance2(IndexDir, DIR, varg1, arg1)
  result = fs_close_index_dir(arg1);
  DATA_PTR(varg1) = NULL;
  if (result < 0) {
    DEBUG_TRACE();
    my_raise(errno);
  }
  return Qnil;
}


static VALUE
yld_fs_close_index_dir(VALUE self)
{
  return rb_fs_close_index_dir(cIndexDir, self);
}


static VALUE
rb_index_dir_close(VALUE self)
{
  return rb_fs_close_index_dir(cIndexDir, self);
}


static VALUE
rb_fs_open_index_dir(VALUE self, VALUE varg1)
{
  dev_t arg1;
  DIR *result;
  VALUE vresult;

  arg1 = NUM2LONG(varg1);
  result = fs_open_index_dir(arg1);
  if (!result) {
    DEBUG_TRACE();
    my_raise(errno);
  }
  vresult = Data_Wrap_Struct(cIndexDir, 0, close_index_dir, result);
  if (rb_block_given_p()) {
    vresult = rb_ensure(RUBY_METHOD_FUNC(rb_yield), vresult,
                        RUBY_METHOD_FUNC(yld_fs_close_index_dir), vresult);
  }
  return vresult;
}


static VALUE
rb_fs_read_index_dir(VALUE self, VALUE varg1)
{
  DIR *arg1;
  dirent_t *result;
  VALUE vresult = Qnil;

  Get_BInstance2(IndexDir, DIR, varg1, arg1)
  if (rb_block_given_p()) {
    while ((result = fs_read_index_dir(arg1))) {
      vresult = rb_yield(my_dirent_t2hash(*result));
    }
  }
  else {
    result = fs_read_index_dir(arg1);
    if (!result) {
      DEBUG_TRACE();
      my_raise(errno);
    }
    vresult = my_dirent_t2hash(*result);
  }
  return vresult;
}


static VALUE
rb_index_dir_read(VALUE self)
{
  return rb_fs_read_index_dir(cIndexDir, self);
}


static VALUE
rb_fs_rewind_index_dir(VALUE self, VALUE varg1)
{
  DIR *arg1;

  Get_BInstance2(IndexDir, DIR, varg1, arg1)
  fs_rewind_index_dir(arg1);
  return Qnil;
}


static VALUE
rb_index_dir_rewind(VALUE self)
{
  return rb_fs_rewind_index_dir(cIndexDir, self);
}


static VALUE
rb_fs_create_index(VALUE self, VALUE varg1, VALUE varg2, VALUE varg3, VALUE varg4)
{
  dev_t arg1;
  char *arg2;
  int arg3, result;
  uint arg4;

  arg1 = NUM2LONG(varg1);
  arg2 = STR2CSTR(varg2);
  arg3 = NUM2LONG(varg3);
  arg4 = NUM2ULONG(varg4);
  result = fs_create_index(arg1, arg2, arg3, arg4);
  if (result < 0) {
    DEBUG_TRACE();
    my_raise(errno);
  }
  return Qnil;
}


static VALUE
rb_fs_remove_index(VALUE self, VALUE varg1, VALUE varg2)
{
  dev_t arg1;
  char *arg2;
  int result;

  arg1 = NUM2LONG(varg1);
  arg2 = STR2CSTR(varg2);
  result = fs_remove_index(arg1, arg2);
  if (result < 0) {
    DEBUG_TRACE();
    my_raise(errno);
  }
  return Qnil;
}


static VALUE
my_index_info2hash(index_info &idx)
{
  VALUE vresult = rb_hash_new();
  rb_hash_aset(vresult, rb_str_new2("type"), ULONG2NUM(idx.type));
  rb_hash_aset(vresult, rb_str_new2("size"), LLONG2NUM(idx.size));
  rb_hash_aset(vresult, rb_str_new2("modification_time"), LONG2NUM(idx.modification_time));
  rb_hash_aset(vresult, rb_str_new2("creation_time"),     LONG2NUM(idx.creation_time));
  rb_hash_aset(vresult, rb_str_new2("uid"), ULONG2NUM(idx.uid));
  rb_hash_aset(vresult, rb_str_new2("gid"), ULONG2NUM(idx.gid));
  return vresult;
}


static VALUE
rb_fs_stat_index(VALUE self, VALUE varg1, VALUE varg2)
{
  dev_t arg1;
  char *arg2;
  index_info arg3;
  int result;
  VALUE vresult;

  arg1 = NUM2LONG(varg1);
  arg2 = STR2CSTR(varg2);
  result = fs_stat_index(arg1, arg2, &arg3);
  if (result < 0) {
    DEBUG_TRACE();
    my_raise(errno);
  }
  vresult = my_index_info2hash(arg3);
  return vresult;
}


void
Init_fs_index()
{
  cIndexDir = rb_define_class_under(mBe, "IndexDir", rb_cObject);
  rb_undef_method(CLASS_OF(cIndexDir), "new");
  rb_define_method(cIndexDir, "close", RUBY_METHOD_FUNC(rb_index_dir_close), 0);
  rb_define_method(cIndexDir, "read", RUBY_METHOD_FUNC(rb_index_dir_read), 0);
  rb_define_method(cIndexDir, "rewind", RUBY_METHOD_FUNC(rb_index_dir_rewind), 0);

  rb_define_module_function(mBe, "fs_open_index_dir", RUBY_METHOD_FUNC(rb_fs_open_index_dir), 1);
  rb_define_module_function(mBe, "fs_close_index_dir", RUBY_METHOD_FUNC(rb_fs_close_index_dir), 1);
  rb_define_module_function(mBe, "fs_read_index_dir", RUBY_METHOD_FUNC(rb_fs_read_index_dir), 1);
  rb_define_module_function(mBe, "fs_rewind_index_dir", RUBY_METHOD_FUNC(rb_fs_rewind_index_dir), 1);

  rb_define_module_function(mBe, "fs_create_index", RUBY_METHOD_FUNC(rb_fs_create_index), 4);
  rb_define_module_function(mBe, "fs_remove_index", RUBY_METHOD_FUNC(rb_fs_remove_index), 2);

  rb_define_module_function(mBe, "fs_stat_index", RUBY_METHOD_FUNC(rb_fs_stat_index), 2);
}
